package ru.amse.smyshlyaev.grapheditor.ui;

import java.awt.*;
import java.util.List;
import java.util.*;
import javax.swing.*;

import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.ui.drawer.*;
import ru.amse.smyshlyaev.grapheditor.ui.tool.defaulttool.*;
import ru.amse.smyshlyaev.grapheditor.ui.tool.ITool;
import ru.amse.smyshlyaev.grapheditor.ui.tool.labeleditor.*;
import ru.amse.smyshlyaev.grapheditor.ui.tool.vertexcreator.*;
import ru.amse.smyshlyaev.grapheditor.ui.tool.edgecreator.*;


public class JGraphComponent extends JComponent {
	private Graph myGraph;
	private VertexDrawer myVertexDrawer;
	private EdgeDrawer myEdgeDrawer;
	private List<ISelectableVertex> myMarkedVertices;
	private IEdge myMarkedEdge;
	private ITool myTool;
	private DefaultToolManager myDefaultTool;
	private VertexLabelEditor myVertexLabelEditor;
	private EdgeLabelEditor myEdgeLabelEditor;
	private VertexCreator myVertexCreator;
	private EdgeCreator myEdgeCreator;
	private StarCreator myStarCreator;

	public JGraphComponent(Graph graph, 
			int width, int height) {
		myGraph = graph;
		myVertexDrawer = new VertexDrawer();
		myEdgeDrawer = new EdgeDrawer();
		myMarkedVertices = new ArrayList<ISelectableVertex>();

		setLayout(new BorderLayout());

		setSize(width, height);
		setPreferredSize(
				new Dimension(width, height)
		);
		this.setLocation(-width / 2, -height / 2);

		myDefaultTool =
     			new DefaultToolManager(this);

		myTool = myDefaultTool;
		
		this.addMouseListener(myTool);
		this.addMouseMotionListener(myTool);
		this.addKeyListener(myTool);
				
		myVertexLabelEditor =
    		new VertexLabelEditor(this);

		myEdgeLabelEditor =
    		new EdgeLabelEditor(this);

		myVertexCreator =
    		new VertexCreator(this);

		myEdgeCreator =
    		new EdgeCreator(this);

		myStarCreator =
    		new StarCreator(this);
	}

	public IGraph getGraph() {
		return Graph.unmodifiableGraph(myGraph);
	}

	public void setGraph(Graph graph) {
		clearMarkedVertices();
		setMarkedEdge(null);
		myGraph = graph;
	}
	
	public void addMarkedVertex(ISelectableVertex v) {
		v.select();
		myMarkedVertices.add(v);
	}
	
	public void addMarkedVertices(List<ISelectableVertex> list) {
		for (ISelectableVertex v : myMarkedVertices) {
			v.select();
		}

		myMarkedVertices.addAll(list);
	}

	public void clearMarkedVertices() {
		for (ISelectableVertex v : myMarkedVertices) {
			v.deselect();
		}
		
		myMarkedVertices.clear();
	}
	
	public void clearMarkedVertex(ISelectableVertex v) {
		v.deselect();
		
		myMarkedVertices.remove(v);
	}

	public void clearMarkedVertices(List<ISelectableVertex> list) {
		for (ISelectableVertex v : list) {
			v.deselect();
		}

		myMarkedVertices.removeAll(list);
	}

	public void setMarkedVertices(List<ISelectableVertex> list) {
		clearMarkedVertices();
		addMarkedVertices(list);
	}
	
	public void removeMarkedVertices() {
		Set<ISelectableVertex> vertices = 
			new HashSet<ISelectableVertex>();
		
		for (ISelectableVertex v : myMarkedVertices) {
			vertices.add(v);
		}
		
		myGraph.removeVertices(myMarkedVertices);
		clearMarkedVertices();
	}
	
	public void removeMarkedVertices(List<ISelectableVertex> list) {
		myGraph.removeVertices(list);
	}
	
	public List<ISelectableVertex> getMarkedVertices() {
		return Collections.unmodifiableList(myMarkedVertices);
	}
	
	public void setMarkedEdge(IEdge e) {
		myMarkedEdge = e;
	}

	public IEdge getMarkedEdge() {
		return myMarkedEdge;
	}
	
	public void addVertex(ISelectableVertex vertex) {
		myGraph.addVertex(vertex);
	}
	
	public void addEdge(IEdge edge) {
		myGraph.addEdge(edge);
	}
	
	public void removeVertex(ISelectableVertex vertex) {
		myGraph.removeVertex(vertex);
	}
	
	public void removeEdge(IEdge edge) {
		myGraph.removeEdge(edge);
	}

	private void setTool(ITool tool) {
		this.removeMouseListener(myTool);
		this.removeMouseMotionListener(myTool);
		this.removeKeyListener(myTool);
		
		myTool = tool;
		
		this.addMouseListener(myTool);
		this.addMouseMotionListener(myTool);
		this.addKeyListener(myTool);
	}

	public void setDefaultTool() {
		setTool(myDefaultTool);
		repaint();
	}
	
	public void editLabel(ISelectableVertex v) {
		setMarkedEdge(null);
		myVertexLabelEditor.setVertex(v);
		setTool(myVertexLabelEditor);
		repaint();
	}
	
	public void editLabel(IEdge e) {
		setMarkedEdge(null);
		myEdgeLabelEditor.setEdge(e);
		setTool(myEdgeLabelEditor);
		repaint();
	}

	public void createVertex() {
		setMarkedEdge(null);
		clearMarkedVertices();
		setTool(myVertexCreator);
	}
	
	public void createEdge() {
		setMarkedEdge(null);
		clearMarkedVertices();
		setTool(myEdgeCreator);
		repaint();
	}

	public void createStar() {
		setMarkedEdge(null);
		clearMarkedVertices();
		setTool(myStarCreator);
	}
	
	private void drawVertexWithEdges(Graphics gc, ISelectableVertex v) {
		Graphics2D g2d = (Graphics2D)(gc);
		
		g2d.setColor(Color.BLACK.brighter());

		myVertexDrawer.setVertex(v);
    	myVertexDrawer.draw(gc);
	
		Set<IEdge> edges = v.getEdges();

		g2d.setColor(Color.GRAY.darker());

    	for (IEdge e : edges) {
    		if (myGraph.hasEdge(e)
    				&& (e.getStart().equals(v))
    				&& (!e.equals(myMarkedEdge))) {
    			myEdgeDrawer.setEdge(e);
	    		myEdgeDrawer.draw(gc);
	    	}
		}		
	}
	
	public void paintComponent(Graphics gc) {
		Set<ISelectableVertex> vertices = myGraph.getVertices();
		
		Graphics2D g2d = (Graphics2D) gc;
		
		g2d.setRenderingHint(
				RenderingHints.KEY_ANTIALIASING, 
		    	RenderingHints.VALUE_ANTIALIAS_ON
		);
		
		g2d.setRenderingHint(
	    		RenderingHints.KEY_TEXT_ANTIALIASING, 
		    	RenderingHints.VALUE_TEXT_ANTIALIAS_ON
		);
		
		requestFocusInWindow();
		
		for (ISelectableVertex v : vertices) {
    		drawVertexWithEdges(gc, v);
		}
	
		myTool.paint(gc);
	}
}